<?php
namespace WpAssetCleanUp;

/**
 * Class OwnAssets
 *
 * These are plugin's own assets (CSS, JS etc.) and they are used only when you're logged in and do not show in the list for unload
 *
 * @package WpAssetCleanUp
 */
class OwnAssets
{
    /**
     * @var bool
     */
    public $loadPluginAssets = false; // default

	/**
	 * @var bool
	 */
	public $isTaxonomyEditPage = false;

	/**
	 *
	 */
	public function init()
    {
        add_action('admin_enqueue_scripts', array($this, 'stylesAndScriptsForAdmin'));

        if (Main::instance()->frontendShow && (! isset($_POST[WPACU_LOAD_ASSETS_REQ_KEY]))) {
            add_action('wp_enqueue_scripts', array($this, 'stylesAndScriptsForPublic'));
        }
    }

    /**
     *
     */
    public function stylesAndScriptsForAdmin()
    {
        global $post;

        if (! Menu::userCanManageAssets()) {
            return;
        }

        $page = (isset($_GET['page'])) ? $_GET['page'] : '';
        $getPostId = (isset($_GET['post'])) ? (int)$_GET['post'] : '';

        // Only load the plugin's assets when they are needed
        // This an example of assets that are correctly loaded in WordPress
        if (isset($post->ID)) {
            $this->loadPluginAssets = true;
        }

        if ($getPostId > 0) {
            $this->loadPluginAssets = true;
        }

        if (strpos($page, WPACU_PLUGIN_NAME) === 0) {
            $this->loadPluginAssets = true;
        }

	    if ($this->isTaxonomyEditPage()) {
		    $this->loadPluginAssets = true;
	    }

        if (! $this->loadPluginAssets) {
            return;
        }

        $this->enqueueAdminStyles();
        $this->enqueueAdminScripts();
    }


    /**
     *
     */
    public function stylesAndScriptsForPublic()
    {
        if (! Menu::userCanManageAssets()) {
            return;
        }

        $this->enqueuePublicStyles();
        $this->enqueuePublicScripts();
    }

    /**
     *
     */
    private function enqueueAdminStyles()
    {
        $styleRelPath = '/assets/style.min.css';
        wp_enqueue_style(WPACU_PLUGIN_NAME . '-style', plugins_url($styleRelPath, WPACU_PLUGIN_FILE), array(), $this->_assetVer($styleRelPath));
    }

    /**
     *
     */
    private function enqueueAdminScripts()
    {
        global $post, $pagenow;

        $page = (isset($_GET['page'])) ? $_GET['page'] : '';

        $getPostId = (isset($_GET['post'])
            && isset($_GET['action'])
            && $_GET['action'] === 'edit'
            && $pagenow == 'post.php')
            ? (int)$_GET['post'] : '';

        $postId = (isset($post->ID)) ? $post->ID : 0;

        if ($getPostId > 0 && $getPostId != $postId) {
            $postId = $getPostId;
        }

        if ($page == WPACU_PLUGIN_NAME.'_home_page' || $postId < 1) {
            $postId = 0; // for home page
        }

        // Not home page (posts list) nor Taxonomy Edit Page? Does it have a post ID?
        // See if the individual post is published to continue
        if ($postId > 0 && (! $this->isTaxonomyEditPage())) {
            $postStatus = get_post_status($postId);

            if (! $postStatus) {
                return;
            }

            // Only for Published Posts
            if ($postStatus != 'publish') {
                return;
            }
        }

        $scriptRelPath = '/assets/script.min.js';

        wp_register_script(
            WPACU_PLUGIN_NAME . '-script',
            plugins_url($scriptRelPath, WPACU_PLUGIN_FILE),
            array('jquery'),
            $this->_assetVer($scriptRelPath)
        );

        // It can also be the front page URL
        $pageUrl = Misc::getPageUrl($postId);

        $wpacuObjectData = array(
	        'plugin_name'  => WPACU_PLUGIN_NAME,
	        'dom_get_type' => Main::$domGetType,
	        'start_del'    => Main::START_DEL,
	        'end_del'      => Main::END_DEL,
	        'ajax_url'     => admin_url('admin-ajax.php'),
	        'post_id'      => $postId, // if any
	        'page_url'     => $pageUrl // post, page, custom post type, homepage etc.
        );

        // [wpacu_lite]
        $submitTicketLink = 'https://wordpress.org/support/plugin/wp-asset-clean-up';
        // [/wpacu_lite]

        $wpacuObjectData['ajax_direct_fetch_error'] = <<<HTML
<div class="ajax-direct-call-error-area">
    <p class="note"><strong>Note:</strong> The checked URL returned an error when fetching the assets via AJAX call. This could be because of a firewall that is blocking the AJAX call, a redirect loop or an error in the script that is retrieving the output which could be due to an incompatibility between the plugin and the WordPress setup you are using.</p>
    <p>Here is the response from the call:</p>

    <table>
        <tr>
            <td width="135"><strong>Status Code Error:</strong></td>
            <td><span class="error-code">{wpacu_status_code_error}</span> * for more information about client and server errors, <a target="_blank" href="https://en.wikipedia.org/wiki/List_of_HTTP_status_codes">check this link</a></td>
        </tr>
        <tr>
            <td valign="top"><strong>Suggestion:</strong></td>
            <td>Select "WP Remote Post" as a method of retrieving the assets from the "Settings" page. If that doesn't fix the issue, just use "Manage in Front-end" option which should always work and <a target="_blank" href="{$submitTicketLink}">submit a ticket</a> about your problem.</td>
        </tr>
        <tr>
            <td><strong>Output:</strong></td>
            <td>{wpacu_output}</td>
        </tr>
    </table>
</div>
HTML;

	    $wpacuObjectData['jquery_migration_disable_confirm_msg'] = __(
            'Make sure to properly test your website if you unload the jQuery migration library.'."\n\n".
            'In some cases, due to old jQuery code triggered from plugins or the theme, unloading this migration library could cause those scripts not to function anymore and break some of the front-end functionality.'."\n\n".
            'If you are not sure about whether activating this option is right or not, it is better to leave it as it is (to be loaded by default) and consult with a developer.'."\n\n".
            'Confirm this action to enable the unloading or cancel to leave it loaded by default.', WPACU_PLUGIN_NAME);

	    $wpacuObjectData['comment_reply_disable_confirm_msg'] = __(
		    'This is worth disabling if you are NOT using the default WordPress comment system (e.g. you are using the website for business purposes, to showcase your products and you are not using it as a blog where people leave comments to your posts).'."\n\n".
		    'If you are not sure about whether activating this option is right or not, it is better to leave it as it is (to be loaded by default).'."\n\n".
		    'Confirm this action to enable the unloading or cancel to leave it loaded by default.', WPACU_PLUGIN_NAME);

        wp_localize_script(
            WPACU_PLUGIN_NAME . '-script',
            'wpacu_object',
            apply_filters('wpacu_object_data', $wpacuObjectData)
        );

        wp_enqueue_script(WPACU_PLUGIN_NAME . '-script');
    }

    /**
     *
     */
    private function enqueuePublicStyles()
    {
        $styleRelPath = '/assets/style.min.css';
        wp_enqueue_style(WPACU_PLUGIN_NAME . '-style', plugins_url($styleRelPath, WPACU_PLUGIN_FILE), array(), $this->_assetVer($styleRelPath));
    }

    /**
     *
     */
    public function enqueuePublicScripts()
    {
        $scriptRelPath = '/assets/script.min.js';
        wp_enqueue_script(WPACU_PLUGIN_NAME . '-script', plugins_url($scriptRelPath, WPACU_PLUGIN_FILE), array('jquery'), $this->_assetVer($scriptRelPath));
    }

    /**
     * @param $relativePath
     * @return bool|false|int|string
     */
    private function _assetVer($relativePath)
    {
        $assetVer = @filemtime(dirname(WPACU_PLUGIN_FILE) . $relativePath);

        if (! $assetVer) {
            $assetVer = date('dmYHi');
        }

        return $assetVer;
    }

	/**
	 *
	 */
	public function codeablePostProject()
    {
        ?>
        <script>
            (function(c,o,d,e,a,b,l){c['CodeableObject']=a;c[a]=c[a]||function(){
                    (c[a].q=c[a].q||[]).push(arguments)},c[a].l=1*new Date();b=o.createElement(d),
                l=o.getElementsByTagName(d)[0];b.async=1;b.src=e;l.parentNode.insertBefore(b,l)
            })(window,document,'script','https://referoo.co/assets/form.js','cdbl');

            cdbl('shortcode', '0JTXB');
            cdbl('render', 'wpacu-get-quote');
        </script>
        <?php
    }

	/**
	 * @return bool
	 */
	public function isTaxonomyEditPage()
    {
        if ((!$this->isTaxonomyEditPage)
            && Main::instance()->wpacuProEnabled()
            && class_exists( '\\WpAssetCleanUpPro\\MainPro' )) {
            $mainPro = new \WpAssetCleanUpPro\MainPro();
	        $this->isTaxonomyEditPage = $mainPro->isTaxonomyEditPage();
        }

        return $this->isTaxonomyEditPage;
    }
}
